/*
   Quadrics Demo.
   Created by the Programming Ace.
   www.UltimateGameProgramming.com

   The purpose of this demo is to create quadric objects.  These objects are used
   to draw shapes.  The quadric objects require the glu.h header file to be
   included.  Once you have a quadric object you can create any shape you want
   by calling the right function.  When you are done with them you can delete the
   objects with a simple function.  To create, draw, then delete a quadric is done like so...

   (assuming we are drawing a sphere)

   GLUquadric *obj = gluNewQuadric();
   gluSphere(obj, 0.8, 25, 25);
   gluDeleteQuadric(obj);

   Thats all there is to it.  The functions are explained below.
*/


// Demo: Quadrics.


#define WIN32_LEAN_AND_MEAN   // This trims down the libraries.
#define VC_LEANMEAN           // Trims even farther.

// All the necessary includes...
#include<windows.h>           // The windows header file.
#include<gl/gl.h>             // Standard opengl include.
#include<gl/glu.h>            // Opengl utilities.


// Function Prototypes
void RenderScene();
bool InitializeGL();
void Shutdown();


// Light properties for the diffuse light, specular light, and light position.
float diffuseLight[] = {0.8f, 0.8f, 0.8f, 1.0f};
float specularLight[] = {1.0f, 1.0f, 1.0f, 1.0f};
float LightPosition[] = {0.0f, 0.0f, 8.0f, 1.0f};

// Here are our quadric objects.  One will draw a triangle and one will draw a sphere.
GLUquadricObj *Triangle = NULL;
GLUquadricObj *Sphere = NULL;


// Global device context.
HDC g_HDC;


void SetupPixelFormat(HDC hDC)
{
   int nPixelFormat;

   static PIXELFORMATDESCRIPTOR pfd = {
         sizeof(PIXELFORMATDESCRIPTOR),   // size of structure.
         1,                               // always 1.
         PFD_DRAW_TO_WINDOW |             // support window
         PFD_SUPPORT_OPENGL |             // support OpenGl
         PFD_DOUBLEBUFFER,                // support double buffering
         PFD_TYPE_RGBA,                   // support RGBA
         16,                              // bit color mode
         0, 0, 0, 0, 0, 0,                // ignore color bits
         0,                               // no alpha buffer
         0,                               // ignore shift bit
         0,                               // no accumulation buffer
         0, 0, 0, 0,                      // ignore accumulation bits.
         16,                              // number of depth buffer bits.
         0,                               // number of stencil buffer bits.
         0,                               // 0 means no auxiliary buffer
         PFD_MAIN_PLANE,                  // The main drawing plane
         0,                               // this is reserved
         0, 0, 0 };                       // layer masks ignored.

   // This chooses the best pixel format and returns index.
   nPixelFormat = ChoosePixelFormat(hDC, &pfd);

   // This set pixel format to device context.
   SetPixelFormat(hDC, nPixelFormat, &pfd);
}


LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   static HGLRC hRC;             // Rendering context.
   static HDC hDC;               // Device context.
   int width, height;            // The window width and height.
	static POINT oldMousePos;     // Last mouse position.
   static POINT currentMousePos; // Current mouse position.
   static bool  isMouseActive;   // Is the left mouse button down.


   switch(message)
      {
         case WM_CREATE:                     // Windows creation.
            hDC = GetDC(hwnd);               // This gets the device context for our window.
            g_HDC = hDC;                     // Assigns the global device context to this one.
            SetupPixelFormat(hDC);           // Call the pixel format function.

            hRC = wglCreateContext(hDC);     // Creates the rendering context.
            wglMakeCurrent(hDC, hRC);        // Makes the rendering context.
            return 0;
            break;


         case WM_CLOSE:                      // Close message.
         case WM_DESTROY:
            wglMakeCurrent(hDC, NULL);
            wglDeleteContext(hRC);           // Deletes the rendering context.

            PostQuitMessage(0);              // Says close the program.
            return 0;
            break;


         case WM_SIZE:                       // re-size message.
            height = HIWORD(lParam);         // This gets the height of the window.
            width = LOWORD(lParam);          // This gets the width of the window.

            if(height==0)                    // we don't want it to be possible for a
               {                             // height of 0.  If it is 0 me make it 1.
                  height = 1;
               }

            glViewport(0, 0, width, height); // resets the viewport to new dimensions.
            glMatrixMode(GL_PROJECTION);     // Sets the projection matrix.
            glLoadIdentity();                // Reset the modelview matrix.

            // calculate the aspect ratio of the window.
            gluPerspective(45.0f, (GLfloat)width/(GLfloat)height, 0.1f, 1000.0f);

            glMatrixMode(GL_MODELVIEW);      // Sets the projection matrix.
            glLoadIdentity();                // Reset the modelview matrix.

            return 0;
            break;


         case WM_KEYDOWN:
		      switch(wParam)
               { 
                  // Close the app if the user presses escape.
		            case VK_ESCAPE:
                     PostQuitMessage(0);
				         break;

                  default:
                     break;
               }
            break;


         default: // Always have a default just in case.
            break;
      }
   
   // What this does is pass all of the unhandled messages to DefWindowProc.
   return (DefWindowProc(hwnd, message, wParam, lParam));
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
   MSG msg;                // A message variable.
   WNDCLASSEX windowClass; // Your Window class.
   HWND hwnd;              // The Window handle.
   bool isFinished;        // This will be used to check if the program is done or not.

   // This is the Window class.
   windowClass.cbSize = sizeof(WNDCLASSEX);        // size of the WNDCLASSEX structure.
   windowClass.style = CS_HREDRAW | CS_VREDRAW;    // style of the window.
   windowClass.lpfnWndProc = WndProc;              // Address to the windows procedure.
   windowClass.cbClsExtra = 0;                     // Extra class information.
   windowClass.cbWndExtra = 0;                     // Extra window information.
   windowClass.hInstance = hInstance;              // Handle of application Instance.
   // Handle of application Icon.
   windowClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
   windowClass.hCursor = LoadCursor(NULL, IDC_ARROW);// mouse cursor
   windowClass.hbrBackground = NULL;               // background color.
   windowClass.lpszMenuName = NULL;                // name of the main menu.
   windowClass.lpszClassName = "UltimateGameProgrammingClass";// window class name.
   // icon when minimized.
   windowClass.hIconSm = LoadIcon(NULL, IDI_APPLICATION);

   // You must register you class with Windows.
   if(!RegisterClassEx(&windowClass)) return 0;

   // Create window...
   hwnd = CreateWindowEx(NULL,// The extended window style.
                        "UltimateGameProgrammingClass",
                        "Quadrics - By the Programming Ace.",// window name.
                        WS_OVERLAPPEDWINDOW | WS_VISIBLE |// The window style.
                        WS_SYSMENU | WS_CLIPCHILDREN |// window style.
                        WS_CLIPSIBLINGS,// window style.
                        0, 0,// window x, y coordinate.
                        640, 480,// window width and height.
                        NULL,// handle to parent window.
                        NULL,// menu.
                        hInstance,// handle to app instance.
                        NULL);   // pointer to window creation data.
   
   // If there was an error with creating the window, then close the program.
   if(!hwnd) return 0;

   ShowWindow(hwnd, SW_SHOW); // This shows the window.
   UpdateWindow(hwnd);        // This forces a paint message.

   isFinished = false;        // False = running, True = not running.

   // If initialize fail (return false), then we don't want the program to run.
   if(!InitializeGL()) isFinished = true;

   // This is the messsage loop.
   while(!isFinished)
      {
         if(PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
            {
               // If a quit message is received then stop rendering and quit the app.
               if(msg.message == WM_QUIT)
                  {
                     isFinished = true;
                  }

               TranslateMessage(&msg);  // Translate any messages.
               DispatchMessage(&msg);   // Dispatch any messages.
            }
         else
            {
               RenderScene();          // Render a frame.
            }
      }

   // Shutdown (free) all resources we used up.
   Shutdown();

   return (int)msg.wParam;      // End the application.
}


bool InitializeGL()
{
   glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

   glShadeModel(GL_SMOOTH);
   glEnable(GL_DEPTH_TEST);   // Enable depth testing.
   glEnable(GL_CULL_FACE);    // Enable culling.
   glFrontFace(GL_CCW);       // Enable counter clockwise culling.
   glEnable(GL_LIGHTING);     // Enable lighting.

   // Set up the light information.
   glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
   glLightfv(GL_LIGHT0, GL_SPECULAR, specularLight);
   glLightfv(GL_LIGHT0, GL_POSITION, LightPosition);
   glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 10.0f);
   glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, 75.0f);

   // Enable the first light.
   glEnable(GL_LIGHT0);

   // Set up the material information for our objects.
   glEnable(GL_COLOR_MATERIAL);
   glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
   glMaterialfv(GL_FRONT, GL_SPECULAR, specularLight);
   glMateriali(GL_FRONT, GL_SHININESS, 128);

   // Here we are creating the quadric objects.  To do so we pretty much
   // call gluNewQuadric() and what is returned by that function is a new
   // quadric object.  Simple as it gets.
   Triangle = gluNewQuadric();
   Sphere = gluNewQuadric();

   return true;
}


void RenderScene()
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);    // Clears the screen.
   glLoadIdentity();                   // Reset modelview matrix for new frame.

   // Translate back 5.
   glTranslatef(0.0f, 0.0f, -5.0f);

   // Here we draw the cylinder to appear to look like a triangle.
   // To draw the object we call gluCylinder() and send to
   // the quadric object, base radius, top radius, height of the object, the total
   // slices, and stacks.  The last two are used to add detail to the object.
   // The first two deal with setting the radius of the bottom of the cylinder and
   // the top.  The third parameter is the height of the object.
   glPushMatrix();

      glTranslatef(-1.0f, -0.7f, 0.0f);
      glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);
      gluCylinder(Triangle, 0.8, 0.0, 2.0, 32, 4);

   glPopMatrix();
   
   // Here we are drawing the sphere.  For the sphere we call gluSphere() and send to
   // it the radius, slices, and stacks.  The first is the size and the last two deal
   // with the detail resolution.
   glPushMatrix();

      glTranslatef(1.0f, 0.0f, 0.0f);
      gluSphere(Sphere, 0.8, 25, 25);

   glPopMatrix();

   SwapBuffers(g_HDC);
}


void Shutdown()
{
   // We call gluDeleteQuadric(GLUquadric *pobj) to delete the quadric object when
   // we are done.
   if(Triangle) gluDeleteQuadric(Triangle);
   if(Sphere) gluDeleteQuadric(Sphere);
}


// Recap:

// To create quadric objects are pretty easy and straight forward.  You first must create
// the object, draw it however you see fit (any shape or style), then delete the object
// when you are done.  That's it.


// Copyright June 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com